@@ -3,7 +3,7 @@ levelh = 13
 
 create_world(levelw, levelh)
 draw_border("st-brownie")
-fill_floor("fl-hay", 0,0, level_width,level_height)
+fill_floor("fl-hay")
 
 
 oxyd(3,3)
@@ -12,5 +12,5 @@ oxyd(level_width-4, 3)
 oxyd(3,level_height-4)
 --oxyd_shuffle()
 
-set_actor("ac-blackball", 10,6.5, {player=0})
+set_actor("ac-blackball", 10,6.5)
 
