@@ -1,23 +1,17 @@
+enigma.AddLevelPack( "levels/m_tutor/index.txt", "Tutorial")
+
 -- in wizard mode add experimental level pack:
 if options.WizardMode > 0 then
     enigma.AddLevelPack( "levels/index_experimental.txt", "Experimental")
 end
 
-
 enigma.AddLevelPack( "levels/index_enigma.txt", "Enigma")
 enigma.AddLevelPack( "levels/index_enigma2.txt", "Enigma 2")
+enigma.AddLevelPack( "levels/index_enigma3.txt", "Enigma 3")
 enigma.AddLevelPack( "levels/index_clones_of_oxyd.txt", "Oxyds")
 enigma.AddLevelPack( "levels/index_esprit.txt", "Esprit")
-enigma.AddLevelPack( "levels/index_sokoban.txt", "Sokoban")
-
---These do not contain enought level to make them good level packs
-
---enigma.AddLevelPack( "levels/index_oxyd.txt", "Oxyd")
---enigma.AddLevelPack( "levels/index_per_oxyd.txt", "Per.Oxyd")
---enigma.AddLevelPack( "levels/index_per_oxyd_linkgame.txt", "Per.Oxyd Linkgame")
---enigma.AddLevelPack( "levels/index_oxyd_magnum.txt", "Oxyd Magnum")
---enigma.AddLevelPack( "levels/index_oxyd_extra.txt", "Oxyd Extra")
 
 -- add test level pack (usually containing levels created by the user)
 enigma.AddLevelPack( "levels/index_user.txt", "User Levels")
+enigma.AddLevelPack( "levels/+index_user.txt", "Arch friendly User Levels")
 
