@@ -4,6 +4,7 @@
 
   LGPL (c) A. Schiffler
 
+  - Better 
 */
 
 #ifdef WIN32
@@ -17,6 +18,209 @@
 #include "SDL_rotozoom.h"
 
 #define MAX(a,b)    (((a) > (b)) ? (a) : (b))
+#define MIN(a,b)    (((a) < (b)) ? (a) : (b))
+
+inline int clamp (int x, int xmin, int xmax)
+{
+    if (x < xmin)
+        return xmin;
+    else if (x > xmax)
+        return xmax;
+    return x;
+}
+
+inline double sinc(double x)
+{
+    if (x == 0)
+        return 1;
+    x *= M_PI;
+    return sin(x)/x;
+}
+
+inline float lanczos2(float x)
+{
+/*     return sinc(x); */
+  if (x < -2 || x > 2)
+    return 0;
+  else
+    return sinc(x) * sinc(x/2);
+/* sinc_table [(int)((x+4)*100. )] * sinc_table[(int)((x/2+4)*100.)]; */
+}
+
+inline float lanczos3(float x)
+{
+/*     return sinc(x); */
+  if (x < -3 || x > 3)
+    return 0;
+  else
+    return sinc(x) * sinc(x/3);
+}
+
+/* inline float lanczos4(float x) */
+/* { */
+/* /\*     return sinc(x); *\/ */
+/*   if (x < -4 || x > 4) */
+/*     return 0; */
+/*   else */
+/*     return sinc_table [(int)((x+4)*100. )] * sinc_table[(int)((x/4+4)*100.)]; */
+/* } */
+
+/* static void */
+/* resample (Uint8 *s, int slen, int spitch, */
+/*           Uint8 *d, int dlen, int dpitch) */
+/* { */
+/*     float f = (float) slen / dlen; */
+
+/*     int i,j; */
+/*     Uint8 *sp; */
+/*     float r, g, b, a; */
+/*     int support = 2; */
+
+/*     for (i=0; i<dlen; ++i) { */
+/*         float x = i*f; */
+/*         float c = 0; */
+/*         float dx = MAX(f, 1.0); */
+/*         float center = (i+0.5)*f; /\* kernel center in the source image *\/ */
+/*         int start = (int) MAX(0, center-support + 0.5); */
+/*         int stop = (int) MIN(slen, center+support+0.5); */
+/*         int n = stop-start; */
+/*         r = g = b = a = 0; */
+/*         sp = s + start*spitch; */
+/*         for (j=0; j<n; ++j) { */
+/*             float ff = lanczos2((j+start-center+0.5)/dx); */
+/*             if (ff != 0) { */
+/*                 r += ff * sp[0]; */
+/*                 g += ff * sp[1]; */
+/*                 b += ff * sp[2]; */
+/*                 a += ff * sp[3]; */
+/*                 c += ff; */
+/*             } */
+/*             sp += spitch; */
+/*         } */
+/*         r /= c; */
+/*         g /= c; */
+/*         b /= c; */
+/*         a /= c; */
+/*         d[0] = (Uint8) clamp (r, 0, 255);  */
+/*         d[1] = (Uint8) clamp (g, 0, 255);  */
+/*         d[2] = (Uint8) clamp (b, 0, 255);  */
+/*         d[3] = (Uint8) clamp (a, 0, 255);  */
+/*         d += dpitch; */
+/*     } */
+/* } */
+
+typedef float (*FilterFunc)(float);
+
+static void
+resample_horiz (SDL_Surface *src, SDL_Surface *dst, FilterFunc filter, int support)
+{
+    int    x, y;
+    int    i,j;
+    Uint8 *sp, *dp;
+    int    r, g, b, a;
+    float  factor  = (float) src->w / dst->w;
+    float  dx      = MAX(factor, 1.0);
+    float *F       = (float*) malloc(sizeof(float)*(2*support+1));
+
+    for (x=0; x<dst->w; ++x) 
+    {
+        float c      = 0;
+        float center = (x+0.5)*factor; /* kernel center in the source image */
+        int   start  = (int) MAX(0, center-support + 0.5);
+        int   stop   = (int) MIN(src->w, center+support+0.5);
+        int   n      = stop-start;
+
+        for (j=0; j<n; ++j) {
+            F[j] = filter((j+start-center+0.5)/dx);
+            c += F[j];
+        }
+        if (c != 0 && c!=1)
+            for (j=0; j<n; ++j)
+                F[j] /= c;
+        dp = (Uint8*)src->pixels + x*4;
+        for (y=0; y<src->h; ++y) {
+            sp = (Uint8*)src->pixels + y*src->pitch + start*4;
+            r = g = b = a = 0;
+            for (j=0; j<n; ++j) {
+                float ff = F[j];
+                r += ff * sp[0];
+                g += ff * sp[1];
+                b += ff * sp[2];
+                a += ff * sp[3];
+                sp += 4;
+            }
+            dp[0] = (Uint8) clamp (r, 0, 255); 
+            dp[1] = (Uint8) clamp (g, 0, 255); 
+            dp[2] = (Uint8) clamp (b, 0, 255); 
+            dp[3] = (Uint8) clamp (a, 0, 255); 
+            dp += src->pitch;
+        }
+    }
+    free(F);
+}
+
+static void
+resample_vert (SDL_Surface *src, SDL_Surface *dst, FilterFunc filter, int support)
+{
+    int    x, y;
+    int    i,j;
+    Uint8 *sp, *dp;
+    int    r, g, b, a;
+    float  factor  = (float) src->h / dst->h;
+    float  dx      = MAX(factor, 1.0);
+    float *F       = (float*) malloc(sizeof(float)*(2*support+1));
+
+    for (y=0; y<dst->h; ++y)
+    {
+        float c      = 0;
+        float center = (y+0.5)*factor; /* kernel center in the source image */
+        int   start  = (int) MAX(0, center-support + 0.5);
+        int   stop   = (int) MIN(src->h, center+support+0.5);
+        int   n      = stop-start;
+
+        for (j=0; j<n; ++j) {
+            F[j] = filter((j+start-center+0.5)/dx);
+            c += F[j];
+        }
+        if (c != 0 && c!=1)
+            for (j=0; j<n; ++j)
+                F[j] /= c;
+        dp = (Uint8*)dst->pixels + y*dst->pitch;
+        for (x=0; x<dst->w; ++x) 
+        {
+            sp = (Uint8*)src->pixels + start*src->pitch + x*4;
+            r = g = b = a = 0;
+            for (j=0; j<n; ++j) {
+                float ff = F[j];
+                r += ff * sp[0];
+                g += ff * sp[1];
+                b += ff * sp[2];
+                a += ff * sp[3];
+                sp += src->pitch;
+            }
+            dp[0] = (Uint8) clamp (r, 0, 255); 
+            dp[1] = (Uint8) clamp (g, 0, 255); 
+            dp[2] = (Uint8) clamp (b, 0, 255); 
+            dp[3] = (Uint8) clamp (a, 0, 255); 
+            dp += 4;
+/*            dp += src->pitch; */
+        }
+    }
+    free(F);
+}
+
+/* FIXME: Can currently only shrink images!!!! */
+
+static void zoomLanczos (SDL_Surface *src, SDL_Surface *dst)
+{
+#if 1
+    resample_horiz(src, dst, lanczos2, 2);
+    resample_vert(src, dst, lanczos2, 2);
+#else
+    resample_horiz(src, dst, lanczos3, 3);
+    resample_vert(src, dst, lanczos3, 3);
+#endif
+}
 
 /*
 
@@ -28,111 +232,164 @@
 
 int zoomSurfaceRGBA(SDL_Surface * src, SDL_Surface * dst, int smooth)
 {
-    int x, y, sx, sy, *sax, *say, *csax, *csay, csx, csy, ex, ey, t1, t2, sstep;
-    tColorRGBA *sp, *csp, *dp;
-    int sgap, dgap;
-
-    /*
-     * Variable setup
-     */
-    if (smooth) {
-	/*
-	 * For interpolation: assume source dimension is one pixel
-	 */
-	/*
-	 * smaller to avoid overflow on right and bottom edge.
-	 */
-
-        if (src->w > dst->w) {
-            // if the image gets shrinked -> use stronger blur :
-            smooth = (int)((double)(src->w)/dst->w+0.5);
-            sx     = (int) (65536.0 * (float) (src->w - smooth) / (float) dst->w);
-            sy     = (int) (65536.0 * (float) (src->h - smooth) / (float) dst->h);
-        }
-        else {
-            smooth = 1;
-            sx = (int) (65536.0 * (float) (src->w - 1) / (float) dst->w);
-            sy = (int) (65536.0 * (float) (src->h - 1) / (float) dst->h);
-        }
-    } else {
+    if (!smooth) {
+        int x, y, sx, sy, *sax, *say, *csax, *csay, csx, csy, ex, ey, t1, t2, sstep;
+        tColorRGBA *sp, *csp, *dp;
+        int sgap, dgap;
 	sx = (int) (65536.0 * (float) src->w / (float) dst->w);
 	sy = (int) (65536.0 * (float) src->h / (float) dst->h);
-    }
 
-    /*
-     * Allocate memory for row increments
-     */
-    if ((sax = (int *) malloc((dst->w + 1) * sizeof(Uint32))) == NULL) {
-	return (-1);
-    }
-    if ((say = (int *) malloc((dst->h + 1) * sizeof(Uint32))) == NULL) {
-	free(sax);
-	return (-1);
-    }
+        /*
+         * Allocate memory for row increments
+         */
+        if ((sax = (int *) malloc((dst->w + smooth) * sizeof(Uint32))) == NULL) {
+            return (-1);
+        }
+        if ((say = (int *) malloc((dst->h + smooth) * sizeof(Uint32))) == NULL) {
+            free(sax);
+            return (-1);
+        }
 
-    /*
-     * Precalculate row increments
-     */
-    csx = 0;
-    csax = sax;
-    for (x = 0; x <= dst->w; x++) {
-	*csax = csx;
-	csax++;
-	csx &= 0xffff;
-	csx += sx;
-    }
-    csy = 0;
-    csay = say;
-    for (y = 0; y <= dst->h; y++) {
-	*csay = csy;
-	csay++;
-	csy &= 0xffff;
-	csy += sy;
-    }
-
-    /*
-     * Pointer setup
-     */
-    sp = csp = (tColorRGBA *) src->pixels;
-    dp = (tColorRGBA *) dst->pixels;
-    sgap = src->pitch - src->w * 4;
-    dgap = dst->pitch - dst->w * 4;
-
-    /*
-     * Switch between interpolating and non-interpolating code
-     */
-    if (smooth) {
-
-	/*
-	 * Interpolating Zoom
-	 */
-
-	/*
-	 * Scan destination
-	 */
-	csay = say;
+        /*
+         * Precalculate row increments
+         */
+        csx = 0;
+        csax = sax;
+        for (x = 0; x < dst->w + smooth; x++) {
+            csax[x] = csx;
+            csx &= 0xffff;
+            csx += sx;
+            if ((csx >> 16) >= src->w)
+                csx -= sx;
+        }
+        csy = 0;
+        csay = say;
+        for (y = 0; y < dst->h + smooth; y++) {
+            csay[y] = csy;
+            csy &= 0xffff;
+            csy += sy;
+            if ((csy >> 16) >= src->h)
+                csy -= sy;
+        }
 
+        /*
+         * Pointer setup
+         */
+        sp = csp = (tColorRGBA *) src->pixels;
+        dp = (tColorRGBA *) dst->pixels;
+        sgap = src->pitch - src->w * 4;
+        dgap = dst->pitch - dst->w * 4;
+    
+        /*
+         * Switch between interpolating and non-interpolating code
+         */
+        csay = say;
 	for (y = 0; y < dst->h; y++) {
-            if (smooth == 1) {
-                /*
-                 * Setup color source pointers
-                 */
+	    sp = csp;
+	    csax = sax;
+	    for (x = 0; x < dst->w; x++, dp++) {
+		*dp = *sp;
+		csax++;
+		sp += (*csax >> 16);
+	    }
+	    csay++;
+	    csp = (tColorRGBA *) ((Uint8 *) csp + (*csay >> 16) * src->pitch);
+	    dp = (tColorRGBA *) ((Uint8 *) dp + dgap);
+	}
+
+        free(sax);
+        free(say);
+        return (0);
+    }
+    else if (dst->w < src->w && dst->h <=src->h) {
+        zoomLanczos (src, dst);
+    }
+    else
+    {
+        int x, y, sx, sy, *sax, *say, *csax, *csay, csx, csy, ex, ey, t1, t2, sstep;
+        tColorRGBA *sp, *csp, *dp;
+        int sgap, dgap;
+
+        /*
+         * Variable setup
+         */
+        if (smooth) {
+            /*
+             * For interpolation: assume source dimension is `smooth'
+             * pixels smaller to avoid overflow on right and bottom edge.
+             */
+            smooth = 1;
+            if (src->w > dst->w) // if the image gets shrinked -> use stronger blur :
+                smooth = (int)((double)(src->w)/dst->w+0.5);
+
+            sx = (int) (65536.0 * (float) (src->w) / (float) dst->w);
+            sy = (int) (65536.0 * (float) (src->h) / (float) dst->h);
+        } else {
+            sx = (int) (65536.0 * (float) src->w / (float) dst->w);
+            sy = (int) (65536.0 * (float) src->h / (float) dst->h);
+        }
+
+        /*
+         * Allocate memory for row increments
+         */
+        if ((sax = (int *) malloc((dst->w + smooth) * sizeof(Uint32))) == NULL) {
+            return (-1);
+        }
+        if ((say = (int *) malloc((dst->h + smooth) * sizeof(Uint32))) == NULL) {
+            free(sax);
+            return (-1);
+        }
+
+        /*
+         * Precalculate row increments
+         */
+        csx = 0;
+        csax = sax;
+        for (x = 0; x < dst->w + smooth; x++) {
+            csax[x] = csx;
+            csx &= 0xffff;
+            csx += sx;
+            if ((csx >> 16) >= src->w)
+                csx -= sx;
+        }
+        csy = 0;
+        csay = say;
+        for (y = 0; y < dst->h + smooth; y++) {
+            csay[y] = csy;
+            csy &= 0xffff;
+            csy += sy;
+            if ((csy >> 16) >= src->h)
+                csy -= sy;
+        }
+
+        /*
+         * Pointer setup
+         */
+        sp = csp = (tColorRGBA *) src->pixels;
+        dp = (tColorRGBA *) dst->pixels;
+        sgap = src->pitch - src->w * 4;
+        dgap = dst->pitch - dst->w * 4;
+    
+        /*
+         * Switch between interpolating and non-interpolating code
+         */
+        csay = say;
+        if (smooth == 1) {
+            /* Interpolating Zoom */
+            for (y = 0; y < dst->h; y++) {
+                /* Setup color source pointers */
                 tColorRGBA *c00, *c01, *c10, *c11;
 
                 c00  = csp;
-                c01  = csp;
-                c01++;
+                c01  = csp+1;
                 c10  = (tColorRGBA *) ((Uint8 *) csp + src->pitch);
-                c11  = c10;
-                c11++;
+                c11  = c10+1;
                 csax = sax;
-                for (x = 0; x < dst->w; x++) {
+                for (x = 0; x < dst->w; x++, dp++) {
 
-                    /*
-                     * Interpolate colors
-                     */
-                    ex = (*csax & 0xffff);
-                    ey = (*csay & 0xffff);
+                    /* Interpolate colors */
+                    ex = *csax & 0xffff;
+                    ey = *csay & 0xffff;
                     t1 = ((((c01->r - c00->r) * ex) >> 16) + c00->r) & 0xff;
                     t2 = ((((c11->r - c10->r) * ex) >> 16) + c10->r) & 0xff;
                     dp->r = (((t2 - t1) * ey) >> 16) + t1;
@@ -146,35 +403,27 @@ int zoomSurfaceRGBA(SDL_Surface * src, SDL_Surface * dst, int smooth)
                     t2 = ((((c11->a - c10->a) * ex) >> 16) + c10->a) & 0xff;
                     dp->a = (((t2 - t1) * ey) >> 16) + t1;
 
-                    /*
-                     * Advance source pointers
-                     */
+                    /* Advance source pointers */
                     csax++;
                     sstep = (*csax >> 16);
                     c00 += sstep;
                     c01 += sstep;
                     c10 += sstep;
                     c11 += sstep;
-                    /*
-                     * Advance destination pointer
-                     */
-                    dp++;
                 }
+                csay++;
+                csp = (tColorRGBA *) ((Uint8 *) csp + (*csay >> 16) * src->pitch);
+                dp  = (tColorRGBA *) ((Uint8 *) dp + dgap);
             }
-            else { // stronger blur for image shrinking
-                /*
-                 * Setup color source pointers
-                 */
-
+        }
+        else if (smooth > 1) { 
+            // stronger blur for image shrinking
+            for (y = 0; y < dst->h; y++) {
                 tColorRGBA *c       = csp;
                 int         smooth2 = smooth*smooth;
                 csax                = sax;
 
-                for (x = 0; x < dst->w; x++) {
-                    /*
-                     * Interpolate colors
-                     */
-
+                for (x = 0; x < dst->w; x++, dp++) {
                     int         xo, yo;
                     tColorRGBA *co    = c;
                     int         r     = 0, g = 0, b = 0, a = 0;
@@ -197,75 +446,35 @@ int zoomSurfaceRGBA(SDL_Surface * src, SDL_Surface * dst, int smooth)
                     dp->b = b/smooth2;
                     dp->a = a/smooth2;
 
-                    /*
-                     * Advance source pointers
-                     */
                     csax++;
                     sstep  = (*csax >> 16);
                     c   += sstep;
-
-                    /*
-                     * Advance destination pointer
-                     */
-                    dp++;
                 }
+                csay++;
+                csp = (tColorRGBA *) ((Uint8 *) csp + (*csay >> 16) * src->pitch);
+                dp  = (tColorRGBA *) ((Uint8 *) dp + dgap);
+            }
+        }
+        else {                      /* Non-Interpolating Zoom */
+            for (y = 0; y < dst->h; y++) {
+                sp = csp;
+                csax = sax;
+                for (x = 0; x < dst->w; x++, dp++) {
+                    *dp = *sp;
+                    csax++;
+                    sp += (*csax >> 16);
+                }
+                csay++;
+                csp = (tColorRGBA *) ((Uint8 *) csp + (*csay >> 16) * src->pitch);
+                dp = (tColorRGBA *) ((Uint8 *) dp + dgap);
             }
-	    /*
-	     * Advance source pointer
-	     */
-	    csay++;
-	    csp = (tColorRGBA *) ((Uint8 *) csp + (*csay >> 16) * src->pitch);
-	    /*
-	     * Advance destination pointers
-	     */
-	    dp  = (tColorRGBA *) ((Uint8 *) dp + dgap);
-	}
 
-    } else {
-
-	/*
-	 * Non-Interpolating Zoom
-	 */
-
-	csay = say;
-	for (y = 0; y < dst->h; y++) {
-	    sp = csp;
-	    csax = sax;
-	    for (x = 0; x < dst->w; x++) {
-		/*
-		 * Draw
-		 */
-		*dp = *sp;
-		/*
-		 * Advance source pointers
-		 */
-		csax++;
-		sp += (*csax >> 16);
-		/*
-		 * Advance destination pointer
-		 */
-		dp++;
-	    }
-	    /*
-	     * Advance source pointer
-	     */
-	    csay++;
-	    csp = (tColorRGBA *) ((Uint8 *) csp + (*csay >> 16) * src->pitch);
-	    /*
-	     * Advance destination pointers
-	     */
-	    dp = (tColorRGBA *) ((Uint8 *) dp + dgap);
-	}
+        }
 
+        free(sax);
+        free(say);
+        return (0);
     }
-
-    /*
-     * Remove temp arrays
-     */
-    free(sax);
-    free(say);
-
-    return (0);
 }
 
 /*
